# -*- coding: iso-8859-1 -*-
"""
This modules contains classes for representing plain context free 
phrase structure grammars.
"""

import operator, symbol, word

class Rule:
	"""This class encapsulates a Rule within a phrase structure
	grammar.  A rule is modeled as a Symbol on the left side and a
	Word on the right side.  It is constructed with a simple string in
	which the left and right sides are separated by a ->.  Attempts to
	construct rules from invalid strings cause a ValueError.
	>>> r = Rule("S -> NP   VP")
	>>> print r
	S -> NP VP
	>>> r.getLeft()
	NonTerminal('S')
	>>> r.getRight()
	Word('NP VP')
	>>> r.applyToLeftmost(word.Word('"S" "is" S was S'))
	Word('S is NP VP was S')
	>>> badRule = Rule("-> whatever")
	Traceback (most recent call last):
	ValueError: Syntax error in -> whatever: Empty left side
	"""
	def __init__(self, rawRule):
		self._parse(rawRule)

	def __str__(self):
		return "%s -> %s"%(self.left, self.right)

	def __repr__(self):
		return "Rule('%s')"%str(self)

	def _parse(self, rawRule):
		"""transforms a string into our internal representation.  Should
		only be called by our constructor and my raise a ValueError.
		"""
		try:
			rawLeft, rawRight = map(str.strip, rawRule.split("->"))
			if not rawLeft:
				raise ValueError, "Empty left side"
			self.left = symbol.NonTerminal(rawLeft)
			self.right = word.Word(rawRight)
		except ValueError, msg:
			raise ValueError, "Syntax error in %s: %s"%(rawRule, msg)

	#nicht hier sondern in prob
	def getProb(self):
		return self.prob

	def getLeft(self):
		"""returns the NonTerminal on the left side of the rule
		"""
		return self.left

	def getRight(self):
		"""returns the right side of the rule as a Word.  This is not
		a copy, so don't change it.
		"""
		return self.right


	def applyToLeftmost(self, word):
		"""applies the rule to the leftmost occurrence of self.left within
		word.
		"""
		return word.replaceLeftmost(self.left, self.right)


class RuleProb(Rule):
        def _parse(self, rawRule):
                """transforms a string into our internal representation.  Should
                only be called by our constructor and my raise a ValueError.
                """
                try:
			prob, rawRule = rawRule.split('|')
                        rawLeft, rawRight = map(str.strip, rawRule.strip().split("->"))
                        if not rawLeft:
                                raise ValueError, "Empty left side"
			self.prob = float(prob.strip())
                        self.left = symbol.NonTerminal(rawLeft)
                        self.right = word.Word(rawRight)
                except ValueError, msg:
                        raise ValueError, "Syntax error in %s: %s"%(rawRule, msg)

	def __str__(self):
		return "P(%s -> %s) = %f"%(self.left, self.right, self.prob)



class Grammar:
	r"""This class encapsulates a phrase structure grammar, i.e. a
	collection of rules.  Basically, for every left side in its rule
	set it maintains a list of rules having this left side.

	It is constructed with the name of a source file containing
	strings suitable for constructing Rules (one on each line, empty
	lines are ignored, one line starts with = and gives the name of
	the start symbol, only the last start symbol decl is honored);
	if that None is passed there, it expects a second argument
	containing a string of the same structure.  Constructing a
	Grammar may raise either an IOError or a ValueError.
	>>> g = Grammar(None, "=S\nS -> NP VP\nNP -> det n\nVP->v\n"
	... 	'v->"walks"\n\n det->"the"\ndet->"a"\nn->"man"\n')
	>>> g.getRulesForNonTerm(symbol.NonTerminal("det"))
	[Rule('det -> the'), Rule('det -> a')]
	>>> g.getRulesForNonTerm(symbol.NonTerminal("X"))
	[]
	>>> g.deriveLeft(symbol.NonTerminal("det"), word.Word('det "det" det n'))
	[Word('the det det n'), Word('a det det n')]
	"""
	def __init__(self, sourceName, ruleString=None, ruleClass=Rule):
		self.ruleDict = {}
		self.ruleClass = ruleClass
		if not sourceName is None:
			ruleString = open(sourceName).read()
		self._parseRules(ruleString)

	def __iter__(self):
		for rule in self.ruleDict:
			if isinstance(self.ruleDict[rule], list):
				for item  in self.ruleDict[rule]:
					yield item
			else:
				yield self.ruleDict[rule]


	def __str__(self):
		return "\n".join(
			map(str, 
				reduce(operator.add, self.ruleDict.values())))

	def _parseRules(self, ruleString):
		"""parses ruleString and fills the ruleDict with the data obtained.
		Only to be called from the constructor.
		"""
		for rawRule in filter(operator.truth,
				[s.strip() for s in ruleString.split("\n")]):
			if rawRule.startswith("="):
				self._setStartSymbol(symbol.NonTerminal(
					rawRule[1:].strip()))
			else:
				self._addRule(self.ruleClass(rawRule))

	def _addRule(self, rule):
		"""inserts the rule into the ruleDict.  Only to be called from
		the constructor.
		"""
		self.ruleDict.setdefault(rule.getLeft(), 
			[]).append(rule)

	def getLexRules(self):
		return [rule for rule in self if len(rule.getRight()) == 1]

	def getRules(self):
		return [rule for rule in self if len(rule.getRight()) != 1]
		

		

	def _setStartSymbol(self, startSymbol):
		self.startSymbol = startSymbol

	def getRulesForNonTerm(self, nonTerm):
		"""returns a list of rules having nonTerm as their left side.
		You get the original, so don't change what you get.
		"""
		return self.ruleDict.get(nonTerm, [])

	def getStartSymbol(self):
		"""returns the start symbol of the grammar or None if none was given.
		"""
		try:
			return self.startSymbol
		except AttributeError:
			return None

	def deriveLeft(self, nonTerm, word, yieldRules=None):
		"""returns the list of Words resulting from applying all rules
		having nonTerm on the left side to the first occurrence of
		nonTerm in word.

		If you set yieldRules to true, you'll get a list of tuples
		(rule, word) instead of a plain list of Words.  This is useful
		if you actually want to do anything with the derivation.
		"""
		res = []
		for rule in self.getRulesForNonTerm(nonTerm):
			if yieldRules:
				res.append((rule, rule.applyToLeftmost(word)))
			else:
				res.append(rule.applyToLeftmost(word))
		return res


def _test():
	import doctest, grammar
	return doctest.testmod(grammar)


if __name__=="__main__":
	#_test()
	g = Grammar("Grammar.txt", ruleClass=RuleProb)
	for rule in g:
		print rule.getLeft(), len(rule.getRight())
