# -*- coding: iso-8859-1 -*-
"""
This module defines the class Symbol and its to child classes Terminal
and NonTerminal.  It is intended to be used in the context of Chomskian
phrase structure grammars.
"""

import re


class Symbol:
	"""This is the base class for both Terminals and NonTerminals.
	You can compare and print Symbols and see what their associated
	strings are.  Two Symbols are equal if and only if they are bot
	of the same class and have the content.  This class is abstract
	in the sense that it probably makes no sense to have "pure"
	Symbols: They're always either Terminal or NonTerminal.
	>>> n = NonTerminal("det")
	>>> t = Terminal("det")
	>>> print n, t, `n`, `t`
	det det NonTerminal('det') Terminal('det')
	>>> int(n==t)
	0
	"""
	def __init__(self, rawStr):
		self.content = rawStr
	
	def getContent(self):
		return self.content
	
	def __str__(self):
		return self.content

	def __cmp__(self, other):
		if self.__class__==other.__class__:
			return cmp(self.content, other.content)
		else:
			return cmp(id(self), id(other))


class Terminal(Symbol):
	"""Making Terminals non-hashable is a safety precaution that
	makes spotting errors in Grammar easier.  There'd be no problem
	enabling hashing if needed.
	"""
	def __repr__(self):
		return "Terminal('%s')"%self.content

	def __str__(self):
		return '%s'%self.content

	def __hash__(self):
		raise TypeError, "Terminal not hashable"


class NonTerminal(Symbol):
	"""Non-terminals need to be hashable since they are used as keys
	in Grammar's ruleDict.
	"""
	def __repr__(self):
		return "NonTerminal('%s')"%self.content

	def __hash__(self):
		return hash(self.content)


def newSymbol(sym):
	"""creates a NonTerminal (sym has the form "[^"]+") or Terminal
	from the string sym and returns it.
	>>> newSymbol('"term"')
	Terminal('term')
	>>> newSymbol("nonterm")
	NonTerminal('nonterm')
	"""
	mat = re.match('"([^"]*)"$', sym)
	if mat:
		return Terminal(mat.group(1))
	return NonTerminal(sym)


def _test():
	import doctest, symbol
	return doctest.testmod(symbol)


if __name__=="__main__":
	_test()
