# -*- coding: iso-8859-1 -*-
"""
This module defines a class Word as a sequence of Symbols (either
Terminals or NonTerminals.
"""

import symbol

class Word:
	"""This class represents a word, i.e., a sequence of Symbols.  It
	is either constructed with a string (in which every symbol ornated
	with quotes (") is considered to be a Terminal, everything else
	is a NonTerminal) or with another word, in which case the current
	Word becomes a copy of the original Word.
	>>> w1 = Word('n "is" det')
	>>> w1.getLeftmostNonTerm()
	NonTerminal('n')
	>>> w2 = w1.replaceLeftmost(symbol.newSymbol('det'), Word('"a"'))
	>>> w2.append(symbol.newSymbol('"preterminal"'))
	>>> print w1, w2
	n is det n is a preterminal
	"""
	def __init__(self, word):
		self.symList = []
		if isinstance(word, Word):
			self.symList = word.symList[:]
		else:
			self._parseRaw(word)
	
	def __str__(self):
		return " ".join(map(str, self.symList))

	def __len__(self):
		return len(self.symList)
	
	def __hash__(self):
		return hash(str(self))
	
	def __cmp__(self, other):
		return cmp(str(self), str(other))

	def __repr__(self):
		return "Word('%s')"%(" ".join(map(str, self.symList)))
	
	def __getitem__(self, index):
		return self.symList[index]

	def _parseRaw(self, rawWord):
		"""parses a string into a word as explained in the class docstring.
		Only to be called from the constructor.
		"""
		for rSym in map(str.strip, rawWord.split()):
			self.symList.append(symbol.newSymbol(rSym))

	def append(self, sym):
		"""appends a symbol to the constructor.  Raises a TypeError if
		sym is not derived from Symbol.
		"""
		if not isinstance(sym, symbol.Symbol):
			raise TypeError, "Only Symbols allowed"
		self.symList.append(sym)

	def isAllTerminal(self):
		"""returns true if all of self's Symbols are Terminals.
		"""
		for s in self.symList:
			if isinstance(s, symbol.NonTerminal):
				return 0
		return 1
	
	def getLeftmostNonTerm(self):
		"""returns the leftmost NonTerminal in self's Symbols.
		"""
		for s in self.symList:
			if isinstance(s, symbol.NonTerminal):
				return s
	
	def replaceLeftmost(self, s, word):
		"""returns a new Word with the leftmost occurrence of the 
		NonTerminal s is replaced with word.
		"""
		applyPos = self.symList.index(s)
		newWord = Word(word=self)
		newWord.symList[applyPos:applyPos+1] = word.symList
		return newWord
	
	def copy(self):
		"""returns a (shallow) copy of self.
		"""
		return Word(word=self)


def _test():
	import doctest, word
	return doctest.testmod(word)


if __name__=="__main__":
	_test()
